from __init__ import *

def xf(n):
    if isinstance(n, int) or isinstance(n, float):
        return Ratio(n)
    if isinstance(n, Ratio) or isinstance(n, Chord) or isinstance(n, Cents) or isinstance(n, Interval94):
        return n
    if isinstance(n, list):
        return Chord(n)
    if isinstance(n, str):
        return Chord([int(numstr) for numstr in n.split(':')])

class Chord:
    def __init__(self, ratio_list):
        self._list = [Ratio(n) for n in ratio_list]
        self._len = len(self._list)
        self.sort()
    def __add__(self, other):
        l = self._list.copy()
        for i in range(self._len):
            l[i] += xf(other)
        return Chord(l)
    def __sub__(self, other):
        return self + (-other)
    def __neg__(self):
        l = self._list.copy()
        for i in range(self._len):
            l[i] *= -1
        return Chord(l)
    def __mul__(self, other):
        l = self._list.copy()
        for i in range(self._len):
            l[i] *= xf(other)
        return Chord(l)
    def __truediv__(self, other):
        l = self._list.copy()
        for i in range(self._len):
            l[i] /= xf(other)
        return Chord(l)
    def __pow__(self, other):
        l = self._list.copy()
        if other == -1:
            return Chord([r**-1 for r in l])
    def __repr__(self):
        s = 'Chord['
        if self.gcd() != 1:
            for i in range(self._len - 1):
                s += repr(self._list[i]) + ', '
            s += repr(self._list[-1])
        else:
            for i in range(self._len - 1):
                s += repr(self._list[i].numerator) + ':'
            s += repr(self._list[-1].numerator)
        return s + ']'
    def __len__(self):
        return len(self._list)
    def __getitem__(self, index):
        return self._list[index]
    def greatest_common(self):
        numerators = [r.numerator for r in self._list]
        denominators = [r.denominators for r in self._list]
    def gcd(self):
        gcd = 1
        for i in range(self._len):
            gcd
            gcd *= self._list[i].denominator
        return gcd
    def gcn(self):
        gcn = 1
        for i in range(self._len):
            gcn
            gcn *= self._list[i].numerator
        return gcn
    def sort(self):
        self._list.sort()
    def colon(self):
        return self * self.gcd() / Monzo(common_factors(self * self.gcd())).ratio()

def Melody(chord):
    return [xf(ratio) for ratio in chord]

def Harmony(chord):
    return [[xf(ratio)] for ratio in chord]

def common_factors(l):
    # Go through monzos and get least common number of each
    common_monzo = l[0].monzo().to_list()
    for r in l[1:]:
        r_monzo = r.monzo()
        for i in range(min([len(common_monzo), len(r_monzo)])):
            if common_monzo[i] > 0 and r_monzo[i] > 0:
                common_monzo[i] = min([common_monzo[i], r_monzo[i]])
            elif common_monzo[i] < 0 and r_monzo[i] < 0:
                common_monzo[i] = max([common_monzo[i], r_monzo[i]])
            else:
                common_monzo[i] = 0
        if len(common_monzo) > i + 1:
            common_monzo = common_monzo[:i+1]
    common_ratio = Monzo(common_monzo).ratio()
    return Monzo(common_monzo)
